<?php

// -------------------------------------------------------------------------- \\
// Post tree system manager - provides interfaces for comments and forums     \\
// -------------------------------------------------------------------------- \\

// Simple InUSE Check
if ( !defined('QF_STARTED') )
        die('Hacking attempt');

if (defined('QF_PSTTREE_LOADED'))
    die('Scripting error');

define('QF_PSTTREE_LOADED', true);


define('QF_PSTTREE_CACHE_PREFIX', 'POSTTREE.');

class QF2_PostTree
{
    var $ptrees = Array();

    function QF2_PostTree()
    {

    }

    function Render_Tree($tid)
    {
        global $QF, $FOX;

        if (!is_numeric($tid))
            return false;

        $tid = (int) $tid;
        if (!isset($this->ptrees[$tid]) && !$this->_Load_Tree($tid))
            return false;

        $data = $this->ptrees[$tid];
        $QF->Run_Module('VIS');
        $QF->Run_Module('UList');

        $QF->VIS->Load_Templates('posttree');
        $root_node = $QF->VIS->Create_Node('FOX_POSTTREE_OUTER');

        $par_nodes = Array(1 => $root_node);
        list($pids, $uids) = qf_2darray_cols($data['ptree'], Array('post_id', 'author_id'));
        $QF->UList->Query_IDs($uids);
        $p_datas = $this->_Load_Posts($pids);

        foreach($data['ptree'] as $pdata)
        {
            $cur_node = $QF->VIS->Add_Node('FOX_POSTTREE_POST', 'SUB_POSTS', $par_nodes[$pdata['t_level']], $pdata);
            $par_nodes[$pdata['t_level']+1] = $cur_node;

            if (!isset($p_datas[$pdata['post_id']]))
                continue;
            $pdata = $p_datas[$pdata['post_id']];
            $node_params = Array(
                'PTEXT' => $pdata['p_text'],
                );
            if ($uinfo = $QF->UList->Get_UserInfo($pdata['author_id']))
            {
                $QF->VIS->Add_Node('USER_INFO_MIN_DIV', 'AUTHOR_INFO', $cur_node, $uinfo /*+ Array('HIDE_ACCESS' => 1)*/);
                $node_params['AUTHOR'] = $uinfo['nick'];
            }
            elseif ($pdata['author'])
                $node_params['AUTHOR_INFO'] = $pdata['author'];
            else
                $node_params['AUTHOR_INFO'] = Lang('NO_DATA');

            $QF->VIS->Add_Data_Array($cur_node, $node_params);

        }

        return $root_node;
    }

    function Create_Tree($class, $data, $params = null)
    {
        global $QF, $FOX;

        $class = strtolower($class);
        $new_data = Array(
            'class' => $class, 'a_key' => '', 'b_key1' => '', 'b_key2' => '', 'b_key3' => '',
            'author' => $QF->User->uname, 'author_id' => $QF->User->UID, 'caption' => '',
            'time' => $QF->Timer->time, 'r_level' => 0, 'w_level' => 1, 'locked' => false, 'marked' => false,
            );

        qf_array_modify($new_data, $params);

        $new_data['data'] = serialize($data);
        $new_data['hash'] = md5($new_data['data']);

        $new_data['r_level'] = min($new_data['r_level'], QF_FOX2_MAXULEVEL);
        $new_data['w_level'] = min($new_data['w_level'], QF_FOX2_MAXULEVEL);
        $new_data['w_level'] = max($new_data['w_level'], $new_data['r_level']);

        if ($tid = $QF->DBase->Do_Insert('pt_roots', $new_data))
        {

            return $tid;
        }
        return false;
    }

    function Add_Post($tid, $text, $parent = 0, $params = null)
    {
        global $QF, $FOX;

        if (!($tree = Get_Tree($tid)))
            return false;

        if (!isset($tinfo['ptree'][$parent]))
            $parent = 0;

        $data = Array(
            'author' => $QF->User->uname, 'author_id' => $QF->User->UID, 'time' => $QF->Timer->time,
            'locked' => false, 'marked' => false,
            );

        qf_array_modify($data, $params);

        $data['root_id'] = $tid;
        $data['parent'] = $parent;
        $data['author_ip'] = $QF->HTTP->IP_int;

        $QF->Run_Module('Parser');
        $QF->Parser->Init_Std_Tags();
        $text = $QF->Parser->Parse($text, QF_BBPARSE_CHECK);
        $hash = md5($text);

        $t_data = Array(
            'o_text' => $text,
            'p_text' => $QF->Parser->Parse($text, QF_BBPARSE_PREP),
            'preparsed' => 1,
            'hash' => $hash,
            );

        // TODO: checking for hash

        $cachename = QF_PSTTREE_CACHE_PREFIX.$tid;
        if ($pid = $QF->DBase->Do_Insert('pt_posts', $data))
        {
            $t_data['post_id'] = $pid;
            $QF->DBase->Do_Insert('pt_ptext', $t_data);
            $QF->Cache->Drop($cachename);
            unset($this->ptrees[$tid]);
            return true;
        }
        return false;
    }

    function Get_Tree($tid)
    {
        global $QF, $FOX;
        if (!is_numeric($tid))
            return false;

        $tid = (int) $tid;
        if (isset($this->ptrees[$tid]) || $this->_Load_Tree($tid))
            return $this->ptrees[$tid];

        return null;
    }

    function _Load_Posts($pids)
    {
        global $QF, $FOX;
        if (!is_array($pids))
            $pids = explode('|', $pids);

        $query = Array('pt_posts'  => Array('fields' => '*', 'where' => Array('post_id' => $pids)),
                       'pt_ptext'  => Array('fields' => '*', 'join' => Array('post_id' => 'post_id')),
                       );

        if ($datas = $QF->DBase->Do_Multitable_Select($query, '', QF_SQL_SELECTALL))
        {
            $QF->Run_Module('Parser');
            $QF->Parser->Init_Std_Tags();
            $got_data = Array();
            foreach ($datas as $data)
            {
                if (!$data['preparsed'])
                {
                    $data['p_text'] = $QF->Parser->Parse($data['o_text'], QF_BBPARSE_PREP);
                    $data['preparsed'] = 1;
                    $QF->DBase->Do_Update('pt_ptext', Array('p_text' => $data['p_text'], 'preparsed' => $data['preparsed']), Array('post_id' => $data['post_id']));
                }

                $data['p_text'] = $QF->Parser->Parse($data['p_text'], QF_BBPARSE_POSTPREP);
                $got_data[$data['post_id']] = $data;
            }
            return $got_data;
        }
        return Array();
    }

    function _Load_Tree($tid)
    {
        global $QF, $FOX;
        if (!is_numeric($tid))
            return false;

        $tid = (int) $tid;
        $cachename = QF_PSTTREE_CACHE_PREFIX.$tid;

        if ($tdata = $QF->Cache->Get($cachename))
        {
            $this->ptrees[$tid] = $tdata;
            return true;
        }
        elseif ($tinfo = $QF->DBase->Do_Select('pt_roots', '*', Array('root_id' => $tid)))
        {
            $pinfos = $QF->DBase->Do_Select_All('pt_posts', Array('post_id', 'parent', 'time', 'deleted'), Array('root_id' => $tid), 'ORDER BY `time` ASC' );
            $i = 1;
            foreach (array_keys($pinfos) as $id)
                $pinfos[$id]['order_id'] = $i++;
            $ptree = qf_2darray_tree($pinfos, 'post_id', 'parent', 0);

            $tinfo['ptree'] = $ptree;

            $this->ptrees[$tid] = $tinfo;
            $QF->Cache->Set($cachename, $tinfo);
            return true;
        }

        return false;
    }

    function _Parse_RootParams($params, $new_params)
    {
        static $my_params = Array('class', 'a_key', 'b_key1', 'b_key2', 'b_key3', 'caption', 'data');
        if (!is_array($params))
            return Array();

        $out_params = Array();
        foreach($my_params as $par_name)
        {
            if (isset($new_params[$par_name]))
                $out_params[$par_name] = $new_params[$par_name];
        }

        if (isset($out_params['data']))
            $out_params['hash'] = md5($out_params['data']);


        $r_level = (isset($new_params['r_level'])) ? $new_params['r_level'] : $params['r_level'];
        $w_level = (isset($new_params['w_level'])) ? $new_params['w_level'] : $params['w_level'];
        $r_level = 0;
    }
}

?>